@@ -25,14 +25,16 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-import sys
 import socket
+import ssl
+import sys
 
+import tornado.gen
+import tornado.httpclient
 import tornado.httpserver
 import tornado.ioloop
 import tornado.iostream
 import tornado.web
-import tornado.httpclient
 
 __all__ = ["ProxyHandler", "run_proxy"]
 
@@ -40,7 +42,7 @@ __all__ = ["ProxyHandler", "run_proxy"]
 class ProxyHandler(tornado.web.RequestHandler):
     SUPPORTED_METHODS = ["GET", "POST", "CONNECT"]
 
-    @tornado.web.asynchronous
+    @tornado.gen.coroutine
     def get(self):
         def handle_response(response):
             if response.error and not isinstance(
@@ -65,6 +67,12 @@ class ProxyHandler(tornado.web.RequestHandler):
                     self.write(response.body)
                 self.finish()
 
+        upstream_ca_certs = self.application.settings.get("upstream_ca_certs", None)
+        ssl_options = None
+
+        if upstream_ca_certs:
+            ssl_options = ssl.create_default_context(cafile=upstream_ca_certs)
+
         req = tornado.httpclient.HTTPRequest(
             url=self.request.uri,
             method=self.request.method,
@@ -72,56 +80,50 @@ class ProxyHandler(tornado.web.RequestHandler):
             headers=self.request.headers,
             follow_redirects=False,
             allow_nonstandard_methods=True,
+            ssl_options=ssl_options,
         )
 
         client = tornado.httpclient.AsyncHTTPClient()
         try:
-            client.fetch(req, handle_response)
+            response = yield client.fetch(req)
+            yield handle_response(response)
         except tornado.httpclient.HTTPError as e:
             if hasattr(e, "response") and e.response:
-                self.handle_response(e.response)
+                yield handle_response(e.response)
             else:
                 self.set_status(500)
                 self.write("Internal server error:\n" + str(e))
                 self.finish()
 
-    @tornado.web.asynchronous
+    @tornado.gen.coroutine
     def post(self):
-        return self.get()
+        yield self.get()
 
-    @tornado.web.asynchronous
+    @tornado.gen.coroutine
     def connect(self):
         host, port = self.request.uri.split(":")
         client = self.request.connection.stream
 
-        def read_from_client(data):
-            upstream.write(data)
-
-        def read_from_upstream(data):
-            client.write(data)
-
-        def client_close(data=None):
-            if upstream.closed():
-                return
-            if data:
-                upstream.write(data)
-            upstream.close()
-
-        def upstream_close(data=None):
-            if client.closed():
-                return
-            if data:
-                client.write(data)
-            client.close()
-
-        def start_tunnel():
-            client.read_until_close(client_close, read_from_client)
-            upstream.read_until_close(upstream_close, read_from_upstream)
-            client.write(b"HTTP/1.0 200 Connection established\r\n\r\n")
+        @tornado.gen.coroutine
+        def start_forward(reader, writer):
+            while True:
+                try:
+                    data = yield reader.read_bytes(4096, partial=True)
+                except tornado.iostream.StreamClosedError:
+                    break
+                if not data:
+                    break
+                writer.write(data)
+            writer.close()
 
         s = socket.socket(socket.AF_INET, socket.SOCK_STREAM, 0)
         upstream = tornado.iostream.IOStream(s)
-        upstream.connect((host, int(port)), start_tunnel)
+        yield upstream.connect((host, int(port)))
+
+        client.write(b"HTTP/1.0 200 Connection established\r\n\r\n")
+        fu1 = start_forward(client, upstream)
+        fu2 = start_forward(upstream, client)
+        yield [fu1, fu2]
 
 
 def run_proxy(port, start_ioloop=True):
