@@ -1,24 +1,32 @@
-import datetime
+import warnings
+
 import mock
 import pytest
-import time
 
-from urllib3.response import HTTPResponse
-from urllib3.packages import six
-from urllib3.packages.six.moves import xrange
-from urllib3.util.retry import Retry, RequestHistory
 from urllib3.exceptions import (
     ConnectTimeoutError,
     InvalidHeader,
     MaxRetryError,
     ReadTimeoutError,
     ResponseError,
+    SSLError,
 )
+from urllib3.packages import six
+from urllib3.packages.six.moves import xrange
+from urllib3.response import HTTPResponse
+from urllib3.util.retry import RequestHistory, Retry
+
+
+@pytest.fixture(scope="function", autouse=True)
+def no_retry_deprecations():
+    with warnings.catch_warnings(record=True) as w:
+        yield
+    assert len([str(x.message) for x in w if "Retry" in str(x.message)]) == 0
 
 
 class TestRetry(object):
     def test_string(self):
-        """ Retry string representation looks the way we expect """
+        """Retry string representation looks the way we expect"""
         retry = Retry()
         assert (
             str(retry)
@@ -42,7 +50,7 @@ class TestRetry(object):
         assert e.value.reason == error
 
     def test_retry_higher_total_loses(self):
-        """ A lower connect timeout than the total is honored """
+        """A lower connect timeout than the total is honored"""
         error = ConnectTimeoutError()
         retry = Retry(connect=2, total=3)
         retry = retry.increment(error=error)
@@ -51,7 +59,7 @@ class TestRetry(object):
             retry.increment(error=error)
 
     def test_retry_higher_total_loses_vs_read(self):
-        """ A lower read timeout than the total is honored """
+        """A lower read timeout than the total is honored"""
         error = ReadTimeoutError(None, "/", "read timed out")
         retry = Retry(read=2, total=3)
         retry = retry.increment(method="GET", error=error)
@@ -60,7 +68,7 @@ class TestRetry(object):
             retry.increment(method="GET", error=error)
 
     def test_retry_total_none(self):
-        """ if Total is none, connect error should take precedence """
+        """if Total is none, connect error should take precedence"""
         error = ConnectTimeoutError()
         retry = Retry(connect=2, total=None)
         retry = retry.increment(error=error)
@@ -77,12 +85,13 @@ class TestRetry(object):
         assert not retry.is_exhausted()
 
     def test_retry_default(self):
-        """ If no value is specified, should retry connects 3 times """
+        """If no value is specified, should retry connects 3 times"""
         retry = Retry()
         assert retry.total == 10
         assert retry.connect is None
         assert retry.read is None
         assert retry.redirect is None
+        assert retry.other is None
 
         error = ConnectTimeoutError()
         retry = Retry(connect=1)
@@ -97,8 +106,22 @@ class TestRetry(object):
         assert Retry(0).raise_on_redirect
         assert not Retry(False).raise_on_redirect
 
+    def test_retry_other(self):
+        """If an unexpected error is raised, should retry other times"""
+        other_error = SSLError()
+        retry = Retry(connect=1)
+        retry = retry.increment(error=other_error)
+        retry = retry.increment(error=other_error)
+        assert not retry.is_exhausted()
+
+        retry = Retry(other=1)
+        retry = retry.increment(error=other_error)
+        with pytest.raises(MaxRetryError) as e:
+            retry.increment(error=other_error)
+        assert e.value.reason == other_error
+
     def test_retry_read_zero(self):
-        """ No second chances on read timeouts, by default """
+        """No second chances on read timeouts, by default"""
         error = ReadTimeoutError(None, "/", "read timed out")
         retry = Retry(read=0)
         with pytest.raises(MaxRetryError) as e:
@@ -117,7 +140,7 @@ class TestRetry(object):
         )
 
     def test_backoff(self):
-        """ Backoff is computed correctly """
+        """Backoff is computed correctly"""
         max_backoff = Retry.BACKOFF_MAX
 
         retry = Retry(total=100, backoff_factor=0.2)
@@ -182,14 +205,14 @@ class TestRetry(object):
         retry = Retry(total=1, status_forcelist=["418"])
         assert not retry.is_retry("GET", status_code=418)
 
-    def test_method_whitelist_with_status_forcelist(self):
-        # Falsey method_whitelist means to retry on any method.
-        retry = Retry(status_forcelist=[500], method_whitelist=None)
+    def test_allowed_methods_with_status_forcelist(self):
+        # Falsey allowed_methods means to retry on any method.
+        retry = Retry(status_forcelist=[500], allowed_methods=None)
         assert retry.is_retry("GET", status_code=500)
         assert retry.is_retry("POST", status_code=500)
 
-        # Criteria of method_whitelist and status_forcelist are ANDed.
-        retry = Retry(status_forcelist=[500], method_whitelist=["POST"])
+        # Criteria of allowed_methods and status_forcelist are ANDed.
+        retry = Retry(status_forcelist=[500], allowed_methods=["POST"])
         assert not retry.is_retry("GET", status_code=500)
         assert retry.is_retry("POST", status_code=500)
 
@@ -237,7 +260,7 @@ class TestRetry(object):
         assert str(e.value.reason) == "conntimeout"
 
     def test_history(self):
-        retry = Retry(total=10, method_whitelist=frozenset(["GET", "POST"]))
+        retry = Retry(total=10, allowed_methods=frozenset(["GET", "POST"]))
         assert retry.history == tuple()
         connection_error = ConnectTimeoutError("conntimeout")
         retry = retry.increment("GET", "/test1", None, connection_error)
@@ -297,6 +320,7 @@ class TestRetry(object):
         new_retry = retry.new()
         assert new_retry.respect_retry_after_header == respect_retry_after_header
 
+    @pytest.mark.freeze_time("2019-06-03 11:00:00", tz_offset=0)
     @pytest.mark.parametrize(
         "retry_after_header,respect_retry_after_header,sleep_duration",
         [
@@ -310,24 +334,29 @@ class TestRetry(object):
             ("Mon, 3 Jun 2019 11:00:00 UTC", True, None),
             # Won't sleep due to current time reached + not respecting header
             ("Mon, 3 Jun 2019 11:00:00 UTC", False, None),
+            # Handle all the formats in RFC 7231 Section 7.1.1.1
+            ("Mon, 03 Jun 2019 11:30:12 GMT", True, 1812),
+            ("Monday, 03-Jun-19 11:30:12 GMT", True, 1812),
+            # Assume that datetimes without a timezone are in UTC per RFC 7231
+            ("Mon Jun  3 11:30:12 2019", True, 1812),
         ],
     )
+    @pytest.mark.parametrize(
+        "stub_timezone",
+        [
+            "UTC",
+            "Asia/Jerusalem",
+            None,
+        ],
+        indirect=True,
+    )
+    @pytest.mark.usefixtures("stub_timezone")
     def test_respect_retry_after_header_sleep(
         self, retry_after_header, respect_retry_after_header, sleep_duration
     ):
         retry = Retry(respect_retry_after_header=respect_retry_after_header)
 
-        # Date header syntax can specify an absolute date; compare this to the
-        # time in the parametrized inputs above.
-        current_time = mock.MagicMock(
-            return_value=time.mktime(
-                datetime.datetime(year=2019, month=6, day=3, hour=11).timetuple()
-            )
-        )
-
-        with mock.patch("time.sleep") as sleep_mock, mock.patch(
-            "time.time", current_time
-        ):
+        with mock.patch("time.sleep") as sleep_mock:
             # for the default behavior, it must be in RETRY_AFTER_STATUS_CODES
             response = HTTPResponse(
                 status=503, headers={"Retry-After": retry_after_header}
