@@ -1,55 +1,51 @@
 import datetime
 import json
 import logging
+import os.path
+import shutil
 import ssl
 import sys
-import shutil
+import tempfile
 import warnings
+from test import (
+    LONG_TIMEOUT,
+    SHORT_TIMEOUT,
+    TARPIT_HOST,
+    notOpenSSL098,
+    notSecureTransport,
+    onlyPy279OrNewer,
+    requires_network,
+    requires_ssl_context_keyfile_password,
+    resolvesLocalhostFQDN,
+)
 
 import mock
 import pytest
+import trustme
 
-from dummyserver.testcase import HTTPSDummyServerTestCase
+import urllib3.util as util
 from dummyserver.server import (
-    CLIENT_CERT,
-    CLIENT_INTERMEDIATE_PEM,
-    CLIENT_NO_INTERMEDIATE_PEM,
-    CLIENT_INTERMEDIATE_KEY,
     DEFAULT_CA,
-    DEFAULT_CA_BAD,
+    DEFAULT_CA_KEY,
     DEFAULT_CERTS,
-    PASSWORD_CLIENT_KEYFILE,
-)
-
-from test import (
-    onlyPy279OrNewer,
-    notSecureTransport,
-    notOpenSSL098,
-    requires_network,
-    requires_ssl_context_keyfile_password,
-    fails_on_travis_gce,
-    requiresTLSv1,
-    requiresTLSv1_1,
-    requiresTLSv1_2,
-    requiresTLSv1_3,
-    TARPIT_HOST,
-    SHORT_TIMEOUT,
-    LONG_TIMEOUT,
+    encrypt_key_pem,
 )
+from dummyserver.testcase import HTTPSDummyServerTestCase
 from urllib3 import HTTPSConnectionPool
-from urllib3.connection import VerifiedHTTPSConnection, RECENT_DATE
+from urllib3.connection import RECENT_DATE, VerifiedHTTPSConnection
 from urllib3.exceptions import (
-    SSLError,
     ConnectTimeoutError,
-    InsecureRequestWarning,
-    SystemTimeWarning,
     InsecurePlatformWarning,
+    InsecureRequestWarning,
     MaxRetryError,
     ProtocolError,
+    SSLError,
+    SystemTimeWarning,
 )
 from urllib3.packages import six
 from urllib3.util.timeout import Timeout
-import urllib3.util as util
+
+from .. import has_alpn
 
 # Retry failed tests
 pytestmark = pytest.mark.flaky
@@ -77,9 +73,59 @@ TLSv1_3_CERTS = DEFAULT_CERTS.copy()
 TLSv1_3_CERTS["ssl_version"] = getattr(ssl, "PROTOCOL_TLS", None)
 
 
+CLIENT_INTERMEDIATE_PEM = "client_intermediate.pem"
+CLIENT_NO_INTERMEDIATE_PEM = "client_no_intermediate.pem"
+CLIENT_INTERMEDIATE_KEY = "client_intermediate.key"
+PASSWORD_CLIENT_KEYFILE = "client_password.key"
+CLIENT_CERT = CLIENT_INTERMEDIATE_PEM
+
+
 class TestHTTPS(HTTPSDummyServerTestCase):
     tls_protocol_name = None
 
+    def tls_protocol_deprecated(self):
+        return self.tls_protocol_name in {"TLSv1", "TLSv1.1"}
+
+    @classmethod
+    def setup_class(cls):
+        super(TestHTTPS, cls).setup_class()
+
+        cls.certs_dir = tempfile.mkdtemp()
+        # Start from existing root CA as we don't want to change the server certificate yet
+        with open(DEFAULT_CA, "rb") as crt, open(DEFAULT_CA_KEY, "rb") as key:
+            root_ca = trustme.CA.from_pem(crt.read(), key.read())
+
+        # Generate another CA to test verification failure
+        bad_ca = trustme.CA()
+        cls.bad_ca_path = os.path.join(cls.certs_dir, "ca_bad.pem")
+        bad_ca.cert_pem.write_to_path(cls.bad_ca_path)
+
+        # client cert chain
+        intermediate_ca = root_ca.create_child_ca()
+        cert = intermediate_ca.issue_cert(u"example.com")
+        encrypted_key = encrypt_key_pem(cert.private_key_pem, b"letmein")
+
+        cert.private_key_pem.write_to_path(
+            os.path.join(cls.certs_dir, CLIENT_INTERMEDIATE_KEY)
+        )
+        encrypted_key.write_to_path(
+            os.path.join(cls.certs_dir, PASSWORD_CLIENT_KEYFILE)
+        )
+        # Write the client cert and the intermediate CA
+        client_cert = os.path.join(cls.certs_dir, CLIENT_INTERMEDIATE_PEM)
+        cert.cert_chain_pems[0].write_to_path(client_cert)
+        cert.cert_chain_pems[1].write_to_path(client_cert, append=True)
+        # Write only the client cert
+        cert.cert_chain_pems[0].write_to_path(
+            os.path.join(cls.certs_dir, CLIENT_NO_INTERMEDIATE_PEM)
+        )
+
+    @classmethod
+    def teardown_class(cls):
+        super(TestHTTPS, cls).teardown_class()
+
+        shutil.rmtree(cls.certs_dir)
+
     def test_simple(self):
         with HTTPSConnectionPool(
             self.host, self.port, ca_certs=DEFAULT_CA
@@ -87,7 +133,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             r = https_pool.request("GET", "/")
             assert r.status == 200, r.data
 
-    @fails_on_travis_gce
+    @resolvesLocalhostFQDN
     def test_dotted_fqdn(self):
         with HTTPSConnectionPool(
             self.host + ".", self.port, ca_certs=DEFAULT_CA
@@ -95,7 +141,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             r = pool.request("GET", "/")
             assert r.status == 200, r.data
 
-    def test_client_intermediate(self, certs_dir):
+    def test_client_intermediate(self):
         """Check that certificate chains work well with client certs
 
         We generate an intermediate CA from the root CA, and issue a client certificate
@@ -106,15 +152,15 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         with HTTPSConnectionPool(
             self.host,
             self.port,
-            key_file=str(certs_dir / CLIENT_INTERMEDIATE_KEY),
-            cert_file=str(certs_dir / CLIENT_INTERMEDIATE_PEM),
+            key_file=os.path.join(self.certs_dir, CLIENT_INTERMEDIATE_KEY),
+            cert_file=os.path.join(self.certs_dir, CLIENT_INTERMEDIATE_PEM),
             ca_certs=DEFAULT_CA,
         ) as https_pool:
             r = https_pool.request("GET", "/certificate")
             subject = json.loads(r.data.decode("utf-8"))
             assert subject["organizationalUnitName"].startswith("Testing cert")
 
-    def test_client_no_intermediate(self, certs_dir):
+    def test_client_no_intermediate(self):
         """Check that missing links in certificate chains indeed break
 
         The only difference with test_client_intermediate is that we don't send the
@@ -123,33 +169,12 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         with HTTPSConnectionPool(
             self.host,
             self.port,
-            cert_file=str(certs_dir / CLIENT_NO_INTERMEDIATE_PEM),
-            key_file=str(certs_dir / CLIENT_INTERMEDIATE_KEY),
+            cert_file=os.path.join(self.certs_dir, CLIENT_NO_INTERMEDIATE_PEM),
+            key_file=os.path.join(self.certs_dir, CLIENT_INTERMEDIATE_KEY),
             ca_certs=DEFAULT_CA,
         ) as https_pool:
-            try:
+            with pytest.raises((SSLError, ProtocolError)):
                 https_pool.request("GET", "/certificate", retries=False)
-            except SSLError as e:
-                if not (
-                    "alert unknown ca" in str(e)
-                    or "invalid certificate chain" in str(e)
-                    or "unknown Cert Authority" in str(e)
-                    or
-                    # https://github.com/urllib3/urllib3/issues/1422
-                    "connection closed via error" in str(e)
-                    or "WSAECONNRESET" in str(e)
-                ):
-                    raise
-            except ProtocolError as e:
-                if not (
-                    "An existing connection was forcibly closed by the remote host"
-                    in str(e)
-                    # Python 3.7.4+
-                    or "WSAECONNRESET" in str(e)  # Windows
-                    or "EPIPE" in str(e)  # macOS
-                    or "ECONNRESET" in str(e)  # OpenSSL
-                ):
-                    raise
 
     @requires_ssl_context_keyfile_password
     def test_client_key_password(self):
@@ -157,21 +182,21 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             self.host,
             self.port,
             ca_certs=DEFAULT_CA,
-            key_file=PASSWORD_CLIENT_KEYFILE,
-            cert_file=CLIENT_CERT,
+            key_file=os.path.join(self.certs_dir, PASSWORD_CLIENT_KEYFILE),
+            cert_file=os.path.join(self.certs_dir, CLIENT_CERT),
             key_password="letmein",
         ) as https_pool:
             r = https_pool.request("GET", "/certificate")
             subject = json.loads(r.data.decode("utf-8"))
-            assert subject["organizationalUnitName"].startswith("Testing server cert")
+            assert subject["organizationalUnitName"].startswith("Testing cert")
 
     @requires_ssl_context_keyfile_password
     def test_client_encrypted_key_requires_password(self):
         with HTTPSConnectionPool(
             self.host,
             self.port,
-            key_file=PASSWORD_CLIENT_KEYFILE,
-            cert_file=CLIENT_CERT,
+            key_file=os.path.join(self.certs_dir, PASSWORD_CLIENT_KEYFILE),
+            cert_file=os.path.join(self.certs_dir, CLIENT_CERT),
             key_password=None,
         ) as https_pool:
             with pytest.raises(MaxRetryError) as e:
@@ -187,26 +212,25 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             conn = https_pool._new_conn()
             assert conn.__class__ == VerifiedHTTPSConnection
 
-            with mock.patch("warnings.warn") as warn:
+            with warnings.catch_warnings(record=True) as w:
                 r = https_pool.request("GET", "/")
                 assert r.status == 200
 
-                # Modern versions of Python, or systems using PyOpenSSL, don't
-                # emit warnings.
-                if (
-                    sys.version_info >= (2, 7, 9)
-                    or util.IS_PYOPENSSL
-                    or util.IS_SECURETRANSPORT
-                ):
-                    assert not warn.called, warn.call_args_list
-                else:
-                    assert warn.called
-                    if util.HAS_SNI:
-                        call = warn.call_args_list[0]
-                    else:
-                        call = warn.call_args_list[1]
-                    error = call[0][1]
-                    assert error == InsecurePlatformWarning
+            # If we're using a deprecated TLS version we can remove 'DeprecationWarning'
+            if self.tls_protocol_deprecated():
+                w = [x for x in w if x.category != DeprecationWarning]
+
+            # Modern versions of Python, or systems using PyOpenSSL, don't
+            # emit warnings.
+            if (
+                sys.version_info >= (2, 7, 9)
+                or util.IS_PYOPENSSL
+                or util.IS_SECURETRANSPORT
+            ):
+                assert w == []
+            else:
+                assert len(w) > 1
+                assert any(x.category == InsecureRequestWarning for x in w)
 
     def test_verified_with_context(self):
         ctx = util.ssl_.create_urllib3_context(cert_reqs=ssl.CERT_REQUIRED)
@@ -272,7 +296,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         # OpenSSL looks up certificates by the hash for their name, see c_rehash
         # TODO infer the bytes using `cryptography.x509.Name.public_bytes`.
         # https://github.com/pyca/cryptography/pull/3236
-        shutil.copyfile(DEFAULT_CA, str(tmpdir / "b6b9ccf9.0"))
+        shutil.copyfile(DEFAULT_CA, str(tmpdir / "81deb5f7.0"))
 
         with HTTPSConnectionPool(
             self.host, self.port, cert_reqs="CERT_REQUIRED", ca_cert_dir=str(tmpdir)
@@ -280,10 +304,15 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             conn = https_pool._new_conn()
             assert conn.__class__ == VerifiedHTTPSConnection
 
-            with mock.patch("warnings.warn") as warn:
+            with warnings.catch_warnings(record=True) as w:
                 r = https_pool.request("GET", "/")
                 assert r.status == 200
-                assert not warn.called, warn.call_args_list
+
+            # If we're using a deprecated TLS version we can remove 'DeprecationWarning'
+            if self.tls_protocol_deprecated():
+                w = [x for x in w if x.category != DeprecationWarning]
+
+            assert w == []
 
     def test_invalid_common_name(self):
         with HTTPSConnectionPool(
@@ -298,7 +327,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
 
     def test_verified_with_bad_ca_certs(self):
         with HTTPSConnectionPool(
-            self.host, self.port, cert_reqs="CERT_REQUIRED", ca_certs=DEFAULT_CA_BAD
+            self.host, self.port, cert_reqs="CERT_REQUIRED", ca_certs=self.bad_ca_path
         ) as https_pool:
             with pytest.raises(MaxRetryError) as e:
                 https_pool.request("GET", "/")
@@ -339,7 +368,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             assert isinstance(cm.value.reason, SSLError)
 
     def test_unverified_ssl(self):
-        """ Test that bare HTTPSConnection can connect, make requests """
+        """Test that bare HTTPSConnection can connect, make requests"""
         with HTTPSConnectionPool(self.host, self.port, cert_reqs=ssl.CERT_NONE) as pool:
             with mock.patch("warnings.warn") as warn:
                 r = pool.request("GET", "/")
@@ -354,7 +383,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
 
     def test_ssl_unverified_with_ca_certs(self):
         with HTTPSConnectionPool(
-            self.host, self.port, cert_reqs="CERT_NONE", ca_certs=DEFAULT_CA_BAD
+            self.host, self.port, cert_reqs="CERT_NONE", ca_certs=self.bad_ca_path
         ) as pool:
             with mock.patch("warnings.warn") as warn:
                 r = pool.request("GET", "/")
@@ -365,6 +394,11 @@ class TestHTTPS(HTTPSDummyServerTestCase):
                 # the unverified warning. Older systems may also emit other
                 # warnings, which we want to ignore here.
                 calls = warn.call_args_list
+
+                # If we're using a deprecated TLS version we can remove 'DeprecationWarning'
+                if self.tls_protocol_deprecated():
+                    calls = [call for call in calls if call[0][1] != DeprecationWarning]
+
                 if (
                     sys.version_info >= (2, 7, 9)
                     or util.IS_PYOPENSSL
@@ -414,7 +448,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             "localhost", self.port, cert_reqs="CERT_REQUIRED", ca_certs=DEFAULT_CA
         ) as https_pool:
             https_pool.assert_fingerprint = (
-                "F2:06:5A:42:10:3F:45:1C:17:FE:E6:07:1E:8A:86:E5"
+                "55:39:BF:70:05:12:43:FA:1F:D1:BF:4E:E8:1B:07:1D"
             )
 
             https_pool.request("GET", "/")
@@ -424,7 +458,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             "localhost", self.port, cert_reqs="CERT_REQUIRED", ca_certs=DEFAULT_CA
         ) as https_pool:
             https_pool.assert_fingerprint = (
-                "92:81:FE:85:F7:0C:26:60:EC:D6:B3:BF:93:CF:F9:71:CC:07:7D:0A"
+                "72:8B:55:4C:9A:FC:1E:88:A1:1C:AD:1B:B2:E7:CC:3E:DB:C8:F9:8A"
             )
             https_pool.request("GET", "/")
 
@@ -433,40 +467,41 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             "localhost", self.port, cert_reqs="CERT_REQUIRED", ca_certs=DEFAULT_CA
         ) as https_pool:
             https_pool.assert_fingerprint = (
-                "C5:4D:0B:83:84:89:2E:AE:B4:58:BB:12:"
-                "F7:A6:C4:76:05:03:88:D8:57:65:51:F3:"
-                "1E:60:B0:8B:70:18:64:E6"
+                "E3:59:8E:69:FF:C5:9F:C7:88:87:44:58:22:7F:90:8D:D9:BC:12:C4:90:79:D5:"
+                "DC:A8:5D:4F:60:40:1E:A6:D2"
             )
             https_pool.request("GET", "/")
 
     def test_assert_invalid_fingerprint(self):
+        def _test_request(pool):
+            with pytest.raises(MaxRetryError) as cm:
+                pool.request("GET", "/", retries=0)
+            assert isinstance(cm.value.reason, SSLError)
+            return cm.value.reason
+
         with HTTPSConnectionPool(
-            "127.0.0.1", self.port, cert_reqs="CERT_REQUIRED", ca_certs=DEFAULT_CA
+            self.host, self.port, cert_reqs="CERT_REQUIRED", ca_certs=DEFAULT_CA
         ) as https_pool:
+
             https_pool.assert_fingerprint = (
                 "AA:AA:AA:AA:AA:AAAA:AA:AAAA:AA:AA:AA:AA:AA:AA:AA:AA:AA:AA"
             )
-
-            def _test_request(pool):
-                with pytest.raises(MaxRetryError) as cm:
-                    pool.request("GET", "/", retries=0)
-                assert isinstance(cm.value.reason, SSLError)
-
-            _test_request(https_pool)
-            https_pool._get_conn()
+            e = _test_request(https_pool)
+            assert "Fingerprints did not match." in str(e)
 
             # Uneven length
             https_pool.assert_fingerprint = "AA:A"
-            _test_request(https_pool)
-            https_pool._get_conn()
+            e = _test_request(https_pool)
+            assert "Fingerprint of invalid length:" in str(e)
 
             # Invalid length
             https_pool.assert_fingerprint = "AA"
-            _test_request(https_pool)
+            e = _test_request(https_pool)
+            assert "Fingerprint of invalid length:" in str(e)
 
     def test_verify_none_and_bad_fingerprint(self):
         with HTTPSConnectionPool(
-            "127.0.0.1", self.port, cert_reqs="CERT_NONE", ca_certs=DEFAULT_CA_BAD
+            "127.0.0.1", self.port, cert_reqs="CERT_NONE", ca_certs=self.bad_ca_path
         ) as https_pool:
             https_pool.assert_fingerprint = (
                 "AA:AA:AA:AA:AA:AAAA:AA:AAAA:AA:AA:AA:AA:AA:AA:AA:AA:AA:AA"
@@ -477,10 +512,10 @@ class TestHTTPS(HTTPSDummyServerTestCase):
 
     def test_verify_none_and_good_fingerprint(self):
         with HTTPSConnectionPool(
-            "127.0.0.1", self.port, cert_reqs="CERT_NONE", ca_certs=DEFAULT_CA_BAD
+            "127.0.0.1", self.port, cert_reqs="CERT_NONE", ca_certs=self.bad_ca_path
         ) as https_pool:
             https_pool.assert_fingerprint = (
-                "92:81:FE:85:F7:0C:26:60:EC:D6:B3:BF:93:CF:F9:71:CC:07:7D:0A"
+                "72:8B:55:4C:9A:FC:1E:88:A1:1C:AD:1B:B2:E7:CC:3E:DB:C8:F9:8A"
             )
             https_pool.request("GET", "/")
 
@@ -494,7 +529,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             "127.0.0.1", self.port, cert_reqs="CERT_REQUIRED", ca_certs=DEFAULT_CA
         ) as https_pool:
             https_pool.assert_fingerprint = (
-                "92:81:FE:85:F7:0C:26:60:EC:D6:B3:BF:93:CF:F9:71:CC:07:7D:0A"
+                "72:8B:55:4C:9A:FC:1E:88:A1:1C:AD:1B:B2:E7:CC:3E:DB:C8:F9:8A"
             )
             https_pool.request("GET", "/")
 
@@ -522,7 +557,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         ) as https_pool:
             https_pool.ca_certs = DEFAULT_CA
             https_pool.assert_fingerprint = (
-                "92:81:FE:85:F7:0C:26:60:EC:D6:B3:BF:93:CF:F9:71:CC:07:7D:0A"
+                "72:8B:55:4C:9A:FC:1E:88:A1:1C:AD:1B:B2:E7:CC:3E:DB:C8:F9:8A"
             )
 
         timeout = Timeout(total=None)
@@ -532,7 +567,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             https_pool.request("GET", "/")
 
     def test_tunnel(self):
-        """ test the _tunnel behavior """
+        """test the _tunnel behavior"""
         timeout = Timeout(total=None)
         with HTTPSConnectionPool(
             self.host, self.port, timeout=timeout, cert_reqs="CERT_NONE"
@@ -591,7 +626,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
                 conn.close()
 
     def test_enhanced_ssl_connection(self):
-        fingerprint = "92:81:FE:85:F7:0C:26:60:EC:D6:B3:BF:93:CF:F9:71:CC:07:7D:0A"
+        fingerprint = "72:8B:55:4C:9A:FC:1E:88:A1:1C:AD:1B:B2:E7:CC:3E:DB:C8:F9:8A"
 
         with HTTPSConnectionPool(
             self.host,
@@ -640,7 +675,13 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             ) as https_pool:
                 https_pool.request(method, url)
 
-        return [x for x in w if not isinstance(x.message, ResourceWarning)]
+        w = [x for x in w if not isinstance(x.message, ResourceWarning)]
+
+        # If we're using a deprecated TLS version we can remove 'DeprecationWarning'
+        if self.tls_protocol_deprecated():
+            w = [x for x in w if x.category != DeprecationWarning]
+
+        return w
 
     def test_set_ssl_version_to_tls_version(self):
         if self.tls_protocol_name is None:
@@ -674,26 +715,129 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             finally:
                 conn.close()
 
+    def test_default_tls_version_deprecations(self):
+        if self.tls_protocol_name is None:
+            pytest.skip("Skipping base test class")
 
-@requiresTLSv1()
+        with HTTPSConnectionPool(
+            self.host, self.port, ca_certs=DEFAULT_CA
+        ) as https_pool:
+            conn = https_pool._get_conn()
+            try:
+                with warnings.catch_warnings(record=True) as w:
+                    conn.connect()
+                    if not hasattr(conn.sock, "version"):
+                        pytest.skip("SSLSocket.version() not available")
+            finally:
+                conn.close()
+
+        if self.tls_protocol_deprecated():
+            assert len(w) == 1
+            assert str(w[0].message) == (
+                "Negotiating TLSv1/TLSv1.1 by default is deprecated "
+                "and will be disabled in urllib3 v2.0.0. Connecting to "
+                "'%s' with '%s' can be enabled by explicitly opting-in "
+                "with 'ssl_version'" % (self.host, self.tls_protocol_name)
+            )
+        else:
+            assert w == []
+
+    def test_no_tls_version_deprecation_with_ssl_version(self):
+        if self.tls_protocol_name is None:
+            pytest.skip("Skipping base test class")
+
+        with HTTPSConnectionPool(
+            self.host, self.port, ca_certs=DEFAULT_CA, ssl_version=util.PROTOCOL_TLS
+        ) as https_pool:
+            conn = https_pool._get_conn()
+            try:
+                with warnings.catch_warnings(record=True) as w:
+                    conn.connect()
+            finally:
+                conn.close()
+
+        assert w == []
+
+    def test_no_tls_version_deprecation_with_ssl_context(self):
+        if self.tls_protocol_name is None:
+            pytest.skip("Skipping base test class")
+
+        with HTTPSConnectionPool(
+            self.host,
+            self.port,
+            ca_certs=DEFAULT_CA,
+            ssl_context=util.ssl_.create_urllib3_context(),
+        ) as https_pool:
+            conn = https_pool._get_conn()
+            try:
+                with warnings.catch_warnings(record=True) as w:
+                    conn.connect()
+            finally:
+                conn.close()
+
+        assert w == []
+
+    @pytest.mark.skipif(sys.version_info < (3, 8), reason="requires python 3.8+")
+    def test_sslkeylogfile(self, tmpdir, monkeypatch):
+        if not hasattr(util.SSLContext, "keylog_filename"):
+            pytest.skip("requires OpenSSL 1.1.1+")
+        keylog_file = tmpdir.join("keylogfile.txt")
+        monkeypatch.setenv("SSLKEYLOGFILE", str(keylog_file))
+        with HTTPSConnectionPool(
+            self.host, self.port, ca_certs=DEFAULT_CA
+        ) as https_pool:
+            r = https_pool.request("GET", "/")
+            assert r.status == 200, r.data
+            assert keylog_file.check(file=1), "keylogfile '%s' should exist" % str(
+                keylog_file
+            )
+            assert keylog_file.read().startswith(
+                "# TLS secrets log file"
+            ), "keylogfile '%s' should start with '# TLS secrets log file'" % str(
+                keylog_file
+            )
+
+    @pytest.mark.parametrize("sslkeylogfile", [None, ""])
+    def test_sslkeylogfile_empty(self, monkeypatch, sslkeylogfile):
+        # Assert that an HTTPS connection doesn't error out when given
+        # no SSLKEYLOGFILE or an empty value (ie 'SSLKEYLOGFILE=')
+        if sslkeylogfile is not None:
+            monkeypatch.setenv("SSLKEYLOGFILE", sslkeylogfile)
+        else:
+            monkeypatch.delenv("SSLKEYLOGFILE", raising=False)
+        with HTTPSConnectionPool(self.host, self.port, ca_certs=DEFAULT_CA) as pool:
+            r = pool.request("GET", "/")
+            assert r.status == 200, r.data
+
+    def test_alpn_default(self):
+        """Default ALPN protocols are sent by default."""
+        if not has_alpn() or not has_alpn(ssl.SSLContext):
+            pytest.skip("ALPN-support not available")
+        with HTTPSConnectionPool(self.host, self.port, ca_certs=DEFAULT_CA) as pool:
+            r = pool.request("GET", "/alpn_protocol", retries=0)
+            assert r.status == 200
+            assert r.data.decode("utf-8") == util.ALPN_PROTOCOLS[0]
+
+
+@pytest.mark.usefixtures("requires_tlsv1")
 class TestHTTPS_TLSv1(TestHTTPS):
     tls_protocol_name = "TLSv1"
     certs = TLSv1_CERTS
 
 
-@requiresTLSv1_1()
+@pytest.mark.usefixtures("requires_tlsv1_1")
 class TestHTTPS_TLSv1_1(TestHTTPS):
     tls_protocol_name = "TLSv1.1"
     certs = TLSv1_1_CERTS
 
 
-@requiresTLSv1_2()
+@pytest.mark.usefixtures("requires_tlsv1_2")
 class TestHTTPS_TLSv1_2(TestHTTPS):
     tls_protocol_name = "TLSv1.2"
     certs = TLSv1_2_CERTS
 
 
-@requiresTLSv1_3()
+@pytest.mark.usefixtures("requires_tlsv1_3")
 class TestHTTPS_TLSv1_3(TestHTTPS):
     tls_protocol_name = "TLSv1.3"
     certs = TLSv1_3_CERTS
